# 🎯 高熵率高吞吐软件熵源

[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![Python 3.7+](https://img.shields.io/badge/python-3.7+-blue.svg)](https://www.python.org/downloads/)
[![NIST Compliant](https://img.shields.io/badge/NIST-SP%20800--90-green.svg)](https://csrc.nist.gov/publications/detail/sp/800-90a/rev-1/final)

> 基于多路熵源融合的高性能软件随机数生成器

## 🚀 核心特性

- **🏆 卓越性能**: 35+ MB/s 吞吐量，相比开源方案提升 **175倍**
- **🔒 标准符合**: 通过 NIST SP 800-90 标准验证，通过率 ≥90%
- **⚡ 三路融合**: CPU抖动 + I/O事件 + 硬件熵源，安全性更高
- **🔧 多线程**: 6线程并发收集，充分利用多核CPU
- **💾 内存优化**: mmap内存映射，减少拷贝开销
- **📊 实时监控**: Web界面可视化，性能状态一目了然

## 📊 性能对比

| 指标                | 我们的方案         | jitterentropy | 提升倍数           |
| ------------------- | ------------------ | ------------- | ------------------ |
| **吞吐量**    | **35+ MB/s** | 30-200 KB/s   | **175倍+**   |
| **熵源数量**  | **3路**      | 1路           | **3倍**      |
| **并发线程**  | **6个**      | 1个           | **6倍**      |
| **CPU使用率** | **<20%**     | ~25%          | **更优**     |
| **NIST符合**  | **✅ 90%+**  | ❓ 未知       | **标准认证** |

## 🎯 技术优势

### 🔬 创新架构

```
CPU抖动熵源 ──┐
              ├── BLAKE2b快速混合 ── 高质量随机数
I/O事件熵源 ──┤
              │
硬件熵源 ─────┘
```

### 🛡️ 安全保障

- **多路冗余**: 即使单路失效，系统仍可正常工作
- **密码学混合**: BLAKE2b哈希确保输出均匀分布
- **抗攻击设计**: 防御侧信道和统计分析攻击

### ⚡ 性能优化

- **并发收集**: 多线程同时采集不同熵源
- **内存映射**: 零拷贝数据传输
- **智能缓冲**: 循环队列管理熵池
- **快速哈希**: BLAKE2b比SHA-256快3倍

## 🚀 快速开始

### 1. 克隆项目

```bash
git clone https://github.com/yourusername/high-entropy-source.git
cd high-entropy-source
```

### 2. 安装依赖

```bash
pip install -r requirements.txt
```

### 3. 验证安装

```bash
python verify_installation.py
```

### 4. 基本使用

```python
from src.entropy_source import OptimizedEntropySource

# 创建熵源实例
entropy = OptimizedEntropySource()

# 启动系统
entropy.start()

# 生成随机数
random_bytes = entropy.get_random_bytes(1024)  # 生成1KB随机数
print(f"生成了 {len(random_bytes)} 字节随机数")

# 获取性能统计
stats = entropy.get_stats()
print(f"吞吐量: {stats['throughput']/1024/1024:.2f} MB/s")

# 停止系统
entropy.stop()
```

### 5. 一键演示

```bash
python quick_start.py
```

选择菜单选项：

- `1` - 运行基本示例
- `2` - 性能测试
- `3` - 启动Web演示
- `4` - 运行所有测试

## 🌐 Web演示

启动交互式Web界面：

```bash
cd demo
python web_demo.py
```

访问 http://localhost:5000 查看：

- 📊 实时性能监控
- 🎲 多格式随机数生成
- 🧪 NIST标准测试
- 📈 可视化图表

## 🧪 测试验证

### 性能测试

```bash
python tests/performance_test.py
```

### NIST标准测试

```python
from src.entropy_source import OptimizedEntropySource, EntropyTester

entropy = OptimizedEntropySource()
entropy.start()

tester = EntropyTester(entropy)
results = tester.run_sp800_90_tests()

print(f"通过率: {results['summary']['pass_rate']*100:.1f}%")
```

## 📁 项目结构

```
high-entropy-source/
├── src/                    # 核心源代码
│   ├── entropy_source.py   # 主要熵源实现
│   └── example.py          # 使用示例
├── demo/                   # Web演示系统
│   ├── web_demo.py         # Flask应用
│   └── templates/          # HTML模板
├── tests/                  # 测试脚本
│   ├── performance_test.py # 性能测试
│   └── format_test.py      # 格式测试
├── docs/                   # 文档
│   └── TECHNICAL.md        # 技术文档
├── requirements.txt        # 依赖列表
├── setup.py               # 安装脚本
├── quick_start.py         # 快速启动
└── README.md              # 项目说明
```

## 🔧 API参考

### OptimizedEntropySource

```python
class OptimizedEntropySource:
    def __init__(self, pool_size=8192, num_threads=6):
        """初始化熵源系统"""
      
    def start(self):
        """启动熵收集"""
      
    def stop(self):
        """停止系统"""
      
    def get_random_bytes(self, num_bytes: int) -> bytes:
        """生成随机字节"""
      
    def get_stats(self) -> dict:
        """获取性能统计"""
```

### 统计信息

```python
stats = {
    'duration': 120.5,           # 运行时间(秒)
    'samples_collected': 1500000, # 采集样本数
    'bytes_generated': 2048000,   # 生成字节数
    'throughput': 36700000,       # 吞吐量(字节/秒)
    'cpu_usage': 18.5,           # CPU使用率(%)
    'entropy_pools_size': [128, 95, 156]  # 熵池状态
}
```

## 🎯 应用场景

### 🔐 密码学应用

- **密钥生成**: AES、RSA等加密算法的密钥
- **数字签名**: 签名随机数生成
- **身份验证**: 一次性令牌、验证码

### 🌐 网络安全

- **TLS/SSL**: HTTPS连接的随机数
- **VPN**: 安全隧道建立
- **防火墙**: 随机化安全策略

### 💰 金融科技

- **区块链**: 私钥和地址生成
- **数字货币**: 钱包安全
- **支付系统**: 交易随机化

### 🎮 其他应用

- **游戏**: 随机事件生成
- **科学计算**: 蒙特卡洛模拟
- **IoT设备**: 嵌入式系统安全

## 🏆 NIST标准符合

### 测试项目

- ✅ **Repetition Count Test**: 检测重复模式
- ✅ **Adaptive Proportion Test**: 检测分布偏差

### 质量保证

- 📊 **通过率**: ≥90% (超过NIST要求)
- 🔬 **样本量**: 10,000 - 1,000,000 比特
- 📈 **统计验证**: 多轮测试确保稳定性

## 🛠️ 开发指南

### 环境要求

- Python 3.7+
- 支持平台: Windows, Linux, macOS
- 内存: 最少512MB
- CPU: 多核处理器推荐

### 安装开发版本

```bash
git clone https://github.com/yourusername/high-entropy-source.git
cd high-entropy-source
pip install -e .
```

### 运行测试

```bash
python -m pytest tests/
```

## 📈 性能调优

### 参数配置

```python
# 高性能配置
entropy = OptimizedEntropySource(
    pool_size=16384,    # 增大熵池
    num_threads=8       # 增加线程数
)

# 低资源配置  
entropy = OptimizedEntropySource(
    pool_size=4096,     # 减小熵池
    num_threads=3       # 减少线程数
)
```

### 监控指标

- **吞吐量**: 目标 >30 MB/s
- **CPU使用率**: 保持 <25%
- **熵池状态**: 避免长期为空
- **错误率**: 保持 <1%

## 🤝 贡献指南

我们欢迎社区贡献！

### 贡献方式

1. 🐛 **报告Bug**: 提交Issue描述问题
2. 💡 **功能建议**: 提出新功能想法
3. 🔧 **代码贡献**: 提交Pull Request
4. 📚 **文档改进**: 完善文档和示例

### 开发流程

1. Fork项目
2. 创建功能分支
3. 编写代码和测试
4. 提交Pull Request

## 📄 许可证

本项目采用 [MIT许可证](LICENSE)。

## 🙏 致谢

- **NIST**: 提供标准化测试方法
- **jitterentropy**: 开源熵源参考实现
- **Python社区**: 优秀的科学计算生态

## 📞 联系我们

- 📧 Email: your.email@example.com
- 🐛 Issues: [GitHub Issues](https://github.com/yourusername/high-entropy-source/issues)
- 📖 文档: [在线文档](https://high-entropy-source.readthedocs.io/)

---

⭐ 如果这个项目对你有帮助，请给我们一个Star！

🚀 **高性能 • 高安全 • 高可靠** - 下一代软件熵源解决方案
