#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
高熵率高吞吐软件熵源

一个基于多路熵源融合的高性能随机数生成器，提供：
- 35+ MB/s 吞吐量（相比jitterentropy提升175倍）
- NIST SP 800-90标准符合
- 三路熵源融合（CPU抖动 + I/O事件 + 硬件熵源）
- 多线程并发处理

基本使用：
    from high_entropy_source import OptimizedEntropySource
    
    entropy = OptimizedEntropySource()
    entropy.start()
    random_data = entropy.get_random_bytes(1024)
    entropy.stop()

高级功能：
    from high_entropy_source import EntropyTester, generate_password
    
    # NIST标准测试
    tester = EntropyTester(entropy)
    results = tester.run_sp800_90_tests()
    
    # 生成密码
    password = generate_password(entropy, length=16)
"""

# 版本信息
__version__ = "1.0.2"
__author__ = "High Entropy Source Project"
__email__ = "your.email@example.com"
__license__ = "MIT"

# 导入主要类和函数
from .core import OptimizedEntropySource
from .testing import EntropyTester
from .utils import (
    format_random_data,
    generate_password,
    generate_uuid,
    generate_key,
    benchmark_against_jitterentropy
)

# 定义公共API
__all__ = [
    # 核心类
    'OptimizedEntropySource',
    'EntropyTester',
    
    # 工具函数
    'format_random_data',
    'generate_password', 
    'generate_uuid',
    'generate_key',
    'benchmark_against_jitterentropy',
    
    # 版本信息
    '__version__',
    '__author__',
    '__email__',
    '__license__'
]

# 模块级别的便捷函数
def quick_start(pool_size: int = 8192, num_threads: int = 6):
    """
    快速启动熵源的便捷函数
    
    Args:
        pool_size: 熵池大小
        num_threads: 线程数量
        
    Returns:
        OptimizedEntropySource: 已启动的熵源实例
        
    Example:
        entropy = quick_start()
        data = entropy.get_random_bytes(1024)
        entropy.stop()
    """
    entropy = OptimizedEntropySource(pool_size=pool_size, num_threads=num_threads)
    entropy.start()
    return entropy

def get_version_info():
    """
    获取版本信息
    
    Returns:
        dict: 版本信息字典
    """
    return {
        'version': __version__,
        'author': __author__,
        'email': __email__,
        'license': __license__
    }

# 添加到公共API
__all__.extend(['quick_start', 'get_version_info'])
