#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
高熵率软件熵源 - 核心模块
包含主要的熵源实现
"""

import os
import sys
import time
import mmap
import ctypes
import hashlib
import threading
import struct
import platform
from collections import deque
from concurrent.futures import ThreadPoolExecutor
from typing import Optional, List, Dict
import numpy as np
import psutil

# 导入原始实现（这里我们需要从src/entropy_source.py复制核心类）
# 为了简化，我只展示关键部分的重构

class OptimizedEntropySource:
    """
    优化的熵源实现
    提供高性能、多路熵源融合的随机数生成
    """
    
    def __init__(self, pool_size: int = 8192, num_threads: int = 6):
        """
        初始化熵源
        
        Args:
            pool_size: 熵池大小
            num_threads: 线程数量
        """
        self.pool_size = pool_size
        self.num_threads = num_threads
        self.running = False
        
        # 熵池 - 三路独立
        self.entropy_pools = [deque(maxlen=pool_size) for _ in range(3)]
        self.pool_locks = [threading.Lock() for _ in range(3)]
        
        # 线程池
        self.executor = ThreadPoolExecutor(max_workers=num_threads)
        self.collectors = []
        
        # 性能统计
        self.stats = {
            'start_time': 0,
            'samples_collected': 0,
            'bytes_generated': 0,
            'throughput': 0,
            'cpu_usage': 0,
            'entropy_pools_size': [0, 0, 0]
        }
        
        # 内存映射
        try:
            self.mmap_file = mmap.mmap(-1, 1024*1024)  # 1MB
        except:
            self.mmap_file = None
            
        # 硬件随机数支持
        self._init_hardware_entropy()
    
    def _init_hardware_entropy(self):
        """初始化硬件熵源"""
        try:
            if platform.system() == "Windows":
                self.rdrand_func = ctypes.windll.kernel32.RtlGenRandom
            else:
                # Linux/Mac 实现
                pass
            self.rdrand_buffer = ctypes.c_uint64()
        except:
            self.rdrand_func = None
            self.rdrand_buffer = None
    
    def start(self) -> bool:
        """
        启动熵源系统
        
        Returns:
            bool: 启动是否成功
        """
        if self.running:
            return False
            
        self.running = True
        self.stats['start_time'] = time.time()
        
        # 启动收集线程
        try:
            self.collectors = [
                self.executor.submit(self._cpu_jitter_collector),
                self.executor.submit(self._io_collector),
                self.executor.submit(self._hardware_collector)
            ]
            return True
        except Exception as e:
            self.running = False
            return False
    
    def stop(self):
        """停止熵源系统"""
        self.running = False
        
        # 等待线程结束
        for collector in self.collectors:
            try:
                collector.result(timeout=1.0)
            except:
                pass
        
        # 清理资源
        if self.mmap_file:
            self.mmap_file.close()
    
    def get_random_bytes(self, num_bytes: int) -> Optional[bytes]:
        """
        生成随机字节
        
        Args:
            num_bytes: 需要的字节数
            
        Returns:
            bytes: 随机字节数据，失败返回None
        """
        if not self.running or num_bytes <= 0:
            return None
            
        output = bytearray()
        start_time = time.time()
        
        while len(output) < num_bytes:
            # 从三个熵池收集数据
            entropy_data = []
            for i in range(3):
                with self.pool_locks[i]:
                    if self.entropy_pools[i]:
                        entropy_data.append(self.entropy_pools[i].popleft())
            
            if entropy_data:
                # 混合熵源
                mixed = self._mix_entropy(entropy_data)
                output.extend(mixed)
            else:
                # 如果熵池为空，等待一下
                time.sleep(0.001)
        
        # 更新统计
        generation_time = time.time() - start_time
        self.stats['bytes_generated'] += num_bytes
        if generation_time > 0:
            self.stats['throughput'] = num_bytes / generation_time
        
        return bytes(output[:num_bytes])
    
    def get_stats(self) -> Dict:
        """
        获取性能统计
        
        Returns:
            dict: 统计信息
        """
        current_time = time.time()
        self.stats['duration'] = current_time - self.stats['start_time']
        self.stats['cpu_usage'] = psutil.cpu_percent(interval=None)
        self.stats['entropy_pools_size'] = [len(pool) for pool in self.entropy_pools]
        
        return self.stats.copy()
    
    def _cpu_jitter_collector(self):
        """CPU抖动熵收集器"""
        samples = np.zeros(1000, dtype=np.uint64)
        
        while self.running:
            try:
                # 批量采集时间戳
                for i in range(1000):
                    start = time.perf_counter_ns()
                    _ = hashlib.blake2b(os.urandom(16)).digest()
                    end = time.perf_counter_ns()
                    samples[i] = end - start
                
                # 提取熵
                diffs = np.diff(samples)
                entropy_bits = diffs & 0x1
                entropy = self._compress_bits(entropy_bits)
                
                # 存入熵池
                with self.pool_locks[0]:
                    self.entropy_pools[0].append(entropy)
                    
                self.stats['samples_collected'] += 1000
                
            except Exception:
                pass
            
            time.sleep(0.01)
    
    def _io_collector(self):
        """I/O事件熵收集器"""
        last_stats = None
        
        while self.running:
            try:
                current_stats = {
                    'disk': psutil.disk_io_counters(),
                    'net': psutil.net_io_counters(),
                    'cpu': psutil.cpu_stats()
                }
                
                if last_stats:
                    # 计算变化量
                    deltas = []
                    if current_stats['disk'] and last_stats['disk']:
                        deltas.extend([
                            current_stats['disk'].read_bytes - last_stats['disk'].read_bytes,
                            current_stats['disk'].write_bytes - last_stats['disk'].write_bytes
                        ])
                    
                    if deltas:
                        entropy = hashlib.blake2b(
                            struct.pack('Q' * len(deltas), *deltas)
                        ).digest()
                        
                        with self.pool_locks[1]:
                            self.entropy_pools[1].append(entropy)
                
                last_stats = current_stats
                
            except Exception:
                pass
            
            time.sleep(0.1)
    
    def _hardware_collector(self):
        """硬件熵收集器"""
        while self.running:
            try:
                # 获取硬件随机数
                hw_entropy = self._get_hardware_entropy(32)
                
                # 结合性能计数器
                perf_data = struct.pack('Q', time.perf_counter_ns())
                
                # 混合
                combined = hw_entropy + perf_data
                entropy = hashlib.blake2b(combined).digest()
                
                with self.pool_locks[2]:
                    self.entropy_pools[2].append(entropy)
                    
            except Exception:
                pass
            
            time.sleep(0.05)
    
    def _get_hardware_entropy(self, size: int) -> bytes:
        """获取硬件熵"""
        if self.rdrand_func and self.rdrand_buffer:
            try:
                result = bytearray()
                for _ in range((size + 7) // 8):
                    if self.rdrand_func(ctypes.byref(self.rdrand_buffer)):
                        result.extend(self.rdrand_buffer.value.to_bytes(8, 'little'))
                return bytes(result[:size])
            except:
                pass
        
        # 回退到os.urandom
        return os.urandom(size)
    
    def _mix_entropy(self, entropy_bytes: List[bytes]) -> bytes:
        """混合多路熵源"""
        if not entropy_bytes:
            return b''
        
        h = hashlib.blake2b()
        for entropy in entropy_bytes:
            h.update(entropy)
        return h.digest()
    
    def _compress_bits(self, bits: np.ndarray) -> bytes:
        """压缩比特数组为字节"""
        # 将比特数组打包为字节
        byte_count = (len(bits) + 7) // 8
        result = bytearray(byte_count)
        
        for i, bit in enumerate(bits):
            if bit:
                byte_idx = i // 8
                bit_idx = i % 8
                result[byte_idx] |= (1 << bit_idx)
        
        return bytes(result)
