#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
高熵率软件熵源 - 测试模块
包含NIST标准测试和性能测试
"""

import math
import time
import numpy as np
from typing import Dict, List, Tuple
from .core import OptimizedEntropySource

class EntropyTester:
    """
    熵源测试器
    提供NIST SP 800-90标准测试和性能基准测试
    """
    
    def __init__(self, entropy_source: OptimizedEntropySource):
        """
        初始化测试器
        
        Args:
            entropy_source: 要测试的熵源实例
        """
        self.entropy_source = entropy_source
    
    def run_sp800_90_tests(self, sample_size: int = 100000) -> Dict:
        """
        运行NIST SP 800-90标准测试
        
        Args:
            sample_size: 测试样本大小（比特数）
            
        Returns:
            dict: 测试结果
        """
        print(f"🧪 运行NIST SP 800-90测试，样本大小: {sample_size} 比特")
        
        # 生成测试数据
        byte_size = (sample_size + 7) // 8
        random_bytes = self.entropy_source.get_random_bytes(byte_size)
        
        if not random_bytes:
            return {'error': '无法生成测试数据'}
        
        # 转换为比特数组
        bits = self._bytes_to_bits(random_bytes)[:sample_size]
        
        # 运行测试
        results = {}
        
        # 1. 重复计数测试
        rep_result = self._repetition_count_test(bits)
        results['repetition_count'] = {
            'name': 'Repetition Count Test',
            'success': rep_result['success'],
            'max_repetition': rep_result['max_repetition'],
            'cutoff': rep_result['cutoff']
        }
        
        # 2. 自适应比例测试
        prop_result = self._adaptive_proportion_test(bits)
        results['adaptive_proportion'] = {
            'name': 'Adaptive Proportion Test', 
            'success': prop_result['success'],
            'max_proportion': prop_result['max_proportion'],
            'threshold': prop_result['threshold']
        }
        
        # 计算总体通过率
        passed_tests = sum(1 for r in results.values() if r['success'])
        total_tests = len(results)
        
        results['summary'] = {
            'total_tests': total_tests,
            'passed_tests': passed_tests,
            'pass_rate': passed_tests / total_tests if total_tests > 0 else 0
        }
        
        return results
    
    def run_performance_test(self, test_sizes: List[int] = None) -> Dict:
        """
        运行性能基准测试
        
        Args:
            test_sizes: 测试数据大小列表（字节）
            
        Returns:
            dict: 性能测试结果
        """
        if test_sizes is None:
            test_sizes = [1024, 10240, 102400, 1048576]  # 1KB, 10KB, 100KB, 1MB
        
        results = {
            'test_sizes': test_sizes,
            'throughput_results': [],
            'generation_times': [],
            'average_throughput': 0
        }
        
        print("🏃 运行性能基准测试...")
        
        for size in test_sizes:
            print(f"  测试 {size:,} 字节...")
            
            # 预热
            self.entropy_source.get_random_bytes(1024)
            
            # 测试
            start_time = time.time()
            data = self.entropy_source.get_random_bytes(size)
            end_time = time.time()
            
            if data:
                generation_time = end_time - start_time
                throughput = size / generation_time if generation_time > 0 else 0
                
                results['generation_times'].append(generation_time)
                results['throughput_results'].append(throughput)
                
                print(f"    时间: {generation_time:.4f}s, 吞吐量: {throughput/1024/1024:.2f} MB/s")
            else:
                print(f"    ❌ 生成失败")
                results['generation_times'].append(0)
                results['throughput_results'].append(0)
        
        # 计算平均吞吐量
        if results['throughput_results']:
            results['average_throughput'] = sum(results['throughput_results']) / len(results['throughput_results'])
        
        return results
    
    def _repetition_count_test(self, bits: List[int]) -> Dict:
        """
        重复计数测试
        检测连续重复的比特
        """
        if not bits:
            return {'success': False, 'max_repetition': 0, 'cutoff': 0}
        
        max_repetition = 0
        current_repetition = 1
        current_bit = bits[0]
        
        for i in range(1, len(bits)):
            if bits[i] == current_bit:
                current_repetition += 1
            else:
                max_repetition = max(max_repetition, current_repetition)
                current_repetition = 1
                current_bit = bits[i]
        
        # 检查最后一段
        max_repetition = max(max_repetition, current_repetition)
        
        # NIST标准：最大重复长度 < log2(n) + 3
        cutoff = math.ceil(math.log2(len(bits))) + 3
        success = max_repetition < cutoff
        
        return {
            'success': success,
            'max_repetition': max_repetition,
            'cutoff': cutoff
        }
    
    def _adaptive_proportion_test(self, bits: List[int], window_size: int = 512) -> Dict:
        """
        自适应比例测试
        检测比特分布偏差
        """
        if len(bits) < window_size:
            return {'success': False, 'max_proportion': 0, 'threshold': 0.67}
        
        max_proportion = 0
        num_windows = len(bits) // window_size
        
        for i in range(num_windows):
            start_idx = i * window_size
            end_idx = start_idx + window_size
            window = bits[start_idx:end_idx]
            
            # 计算1的比例
            proportion = sum(window) / len(window)
            max_proportion = max(max_proportion, proportion)
        
        # NIST标准：任何窗口中1的比例不能超过67%
        threshold = 0.67
        success = max_proportion < threshold
        
        return {
            'success': success,
            'max_proportion': max_proportion,
            'threshold': threshold
        }
    
    def _bytes_to_bits(self, data: bytes) -> List[int]:
        """将字节数据转换为比特列表"""
        bits = []
        for byte in data:
            for i in range(8):
                bits.append((byte >> i) & 1)
        return bits
