#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
高熵率软件熵源 - 工具模块
包含数据格式化和实用工具函数
"""

import string
import random
import uuid
import base64
from typing import Union, List

def format_random_data(data: bytes, format_type: str, length: int) -> str:
    """
    格式化随机数据为不同类型
    
    Args:
        data: 原始随机字节
        format_type: 格式类型
        length: 输出长度
        
    Returns:
        str: 格式化后的数据
    """
    if not data:
        return "无数据"
    
    # 使用随机字节作为种子
    random.seed(int.from_bytes(data[:4], 'big'))
    
    if format_type == 'password':
        # 生成强密码
        chars = string.ascii_letters + string.digits + "!@#$%^&*()_+-=[]{}|;:,.<>?"
        return ''.join(random.choice(chars) for _ in range(length))
    
    elif format_type == 'letters':
        # 只生成字母
        chars = string.ascii_letters
        return ''.join(random.choice(chars) for _ in range(length))
    
    elif format_type == 'numbers':
        # 只生成数字
        return ''.join(str(random.randint(0, 9)) for _ in range(length))
    
    elif format_type == 'alphanumeric':
        # 字母数字混合
        chars = string.ascii_letters + string.digits
        return ''.join(random.choice(chars) for _ in range(length))
    
    elif format_type == 'symbols':
        # 包含各种符号
        chars = string.ascii_letters + string.digits + string.punctuation
        return ''.join(random.choice(chars) for _ in range(length))
    
    elif format_type == 'uuid':
        # 生成UUID
        uuids = []
        for i in range(length):
            uuid_bytes = data[(i * 16) % len(data):(i * 16 + 16) % len(data)]
            if len(uuid_bytes) < 16:
                uuid_bytes += data[:16 - len(uuid_bytes)]
            uuids.append(str(uuid.UUID(bytes=uuid_bytes[:16])))
        return '\n'.join(uuids)
    
    elif format_type == 'base64':
        # Base64编码
        encoded = base64.b64encode(data[:length]).decode('utf-8')
        lines = [encoded[i:i+64] for i in range(0, len(encoded), 64)]
        return '\n'.join(lines)
    
    else:  # hex
        # 十六进制格式
        hex_str = data[:length].hex().upper()
        lines = [hex_str[i:i+32] for i in range(0, len(hex_str), 32)]
        formatted_lines = []
        for i, line in enumerate(lines):
            spaced_line = ' '.join(line[j:j+2] for j in range(0, len(line), 2))
            formatted_lines.append(f"{i*16:08X}: {spaced_line}")
        return '\n'.join(formatted_lines)

def generate_password(entropy_source, length: int = 12, 
                     include_symbols: bool = True) -> str:
    """
    生成安全密码
    
    Args:
        entropy_source: 熵源实例
        length: 密码长度
        include_symbols: 是否包含符号
        
    Returns:
        str: 生成的密码
    """
    random_bytes = entropy_source.get_random_bytes(16)
    if not random_bytes:
        return ""
    
    chars = string.ascii_letters + string.digits
    if include_symbols:
        chars += "!@#$%^&*()_+-="
    
    random.seed(int.from_bytes(random_bytes[:4], 'big'))
    return ''.join(random.choice(chars) for _ in range(length))

def generate_uuid(entropy_source) -> str:
    """
    生成UUID
    
    Args:
        entropy_source: 熵源实例
        
    Returns:
        str: UUID字符串
    """
    random_bytes = entropy_source.get_random_bytes(16)
    if not random_bytes:
        return ""
    
    return str(uuid.UUID(bytes=random_bytes))

def generate_key(entropy_source, key_size: int = 32) -> str:
    """
    生成加密密钥
    
    Args:
        entropy_source: 熵源实例
        key_size: 密钥大小（字节）
        
    Returns:
        str: 十六进制密钥
    """
    key_bytes = entropy_source.get_random_bytes(key_size)
    if not key_bytes:
        return ""
    
    return key_bytes.hex().upper()

def benchmark_against_jitterentropy(entropy_source, 
                                   test_sizes: List[int] = None) -> dict:
    """
    与jitterentropy进行性能对比
    
    Args:
        entropy_source: 我们的熵源实例
        test_sizes: 测试大小列表
        
    Returns:
        dict: 对比结果
    """
    if test_sizes is None:
        test_sizes = [1024, 10240, 102400]  # 1KB, 10KB, 100KB
    
    results = {
        'our_performance': [],
        'jitterentropy_estimated': [],
        'improvement_factor': []
    }
    
    # jitterentropy的估计性能（基于文献数据）
    jitterentropy_throughput = 200 * 1024  # 200 KB/s (最好情况)
    
    for size in test_sizes:
        # 测试我们的性能
        import time
        start_time = time.time()
        data = entropy_source.get_random_bytes(size)
        end_time = time.time()
        
        if data:
            our_time = end_time - start_time
            our_throughput = size / our_time if our_time > 0 else 0
            
            # jitterentropy估计时间
            jitter_time = size / jitterentropy_throughput
            
            # 改进倍数
            improvement = our_throughput / jitterentropy_throughput if jitterentropy_throughput > 0 else 0
            
            results['our_performance'].append({
                'size': size,
                'time': our_time,
                'throughput': our_throughput
            })
            
            results['jitterentropy_estimated'].append({
                'size': size,
                'time': jitter_time,
                'throughput': jitterentropy_throughput
            })
            
            results['improvement_factor'].append(improvement)
    
    return results
