#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
测试原始V1.0.py的性能，验证真实吞吐量
"""

import time
import importlib.util

def load_entropy_module():
    """动态加载熵源模块"""
    try:
        import sys
        import os
        # 添加src目录到路径
        src_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'src')
        sys.path.insert(0, src_path)

        spec = importlib.util.spec_from_file_location("entropy_module", os.path.join(src_path, "entropy_source.py"))
        entropy_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(entropy_module)
        return entropy_module
    except Exception as e:
        print(f"❌ 无法加载熵源模块: {e}")
        print("请确保src/entropy_source.py文件存在")
        return None

def test_original_performance():
    """测试原始代码性能"""
    print("🚀 测试原始V1.0.py的性能")
    print("=" * 50)
    
    entropy_module = load_entropy_module()
    if not entropy_module:
        return
    
    # 创建熵源实例
    print("✓ 创建熵源实例...")
    entropy_source = entropy_module.OptimizedEntropySource()
    
    # 启动熵源
    print("✓ 启动熵源系统...")
    entropy_source.start()
    
    # 预热阶段 - 让熵池积累数据
    print("✓ 预热阶段 (10秒)...")
    for i in range(10):
        time.sleep(1)
        stats = entropy_source.get_stats()
        print(f"  预热 {i+1}/10 - 采集样本: {stats['samples_collected']:,}, 熵池: {stats['entropy_pools_size']}")
    
    print("\n📊 开始性能测试...")
    
    # 测试不同大小的数据生成
    test_sizes = [
        (1024, "1 KB"),
        (4096, "4 KB"), 
        (16384, "16 KB"),
        (65536, "64 KB"),
        (262144, "256 KB"),
        (1048576, "1 MB")
    ]
    
    print("\n数据量".ljust(10) + "生成时间".ljust(12) + "吞吐量".ljust(15) + "状态")
    print("-" * 50)
    
    for size, size_str in test_sizes:
        start_time = time.time()
        random_bytes = entropy_source.get_random_bytes(size)
        generation_time = time.time() - start_time
        
        if random_bytes:
            throughput = len(random_bytes) / generation_time if generation_time > 0 else 0
            throughput_mb = throughput / (1024 * 1024)
            print(f"{size_str:<10}{generation_time:.3f}s".ljust(12) + f"{throughput_mb:.2f} MB/s".ljust(15) + "✅ 成功")
        else:
            print(f"{size_str:<10}超时".ljust(12) + "0 MB/s".ljust(15) + "❌ 失败")
        
        # 短暂等待让熵池恢复
        time.sleep(0.5)
    
    # 连续性能测试
    print(f"\n🔄 连续性能测试 (生成10MB数据)...")
    total_size = 10 * 1024 * 1024  # 10MB
    chunk_size = 64 * 1024  # 64KB chunks
    
    chunks = []
    start_time = time.time()
    total_collected = 0
    failed_attempts = 0
    
    while total_collected < total_size:
        chunk = entropy_source.get_random_bytes(chunk_size)
        if chunk:
            chunks.append(chunk)
            total_collected += len(chunk)
            
            # 显示进度
            progress = (total_collected / total_size) * 100
            if int(progress) % 10 == 0 and len(chunks) % 10 == 1:
                print(f"  进度: {progress:.1f}% ({total_collected/1024/1024:.1f} MB)")
        else:
            failed_attempts += 1
            if failed_attempts > 10:
                print("  ❌ 连续失败次数过多，停止测试")
                break
            time.sleep(0.1)  # 等待熵池恢复
    
    duration = time.time() - start_time
    
    if total_collected > 0:
        throughput = total_collected / duration
        throughput_mb = throughput / (1024 * 1024)
        
        print(f"\n📈 连续性能测试结果:")
        print(f"  总数据量: {total_collected/1024/1024:.2f} MB")
        print(f"  测试时间: {duration:.2f} 秒")
        print(f"  平均吞吐量: {throughput_mb:.2f} MB/s")
        print(f"  失败次数: {failed_attempts}")
    
    # 获取最终统计
    final_stats = entropy_source.get_stats()
    print(f"\n📊 最终系统统计:")
    print(f"  运行时间: {final_stats['duration']:.2f} 秒")
    print(f"  总采集样本: {final_stats['samples_collected']:,}")
    print(f"  总生成字节: {final_stats['bytes_generated']:,}")
    print(f"  系统吞吐量: {final_stats['throughput']/1024/1024:.2f} MB/s")
    print(f"  CPU使用率: {final_stats['cpu_usage']:.1f}%")
    print(f"  熵池状态: {final_stats['entropy_pools_size']}")
    
    # 停止熵源
    print("\n✓ 停止熵源系统...")
    entropy_source.stop()
    
    print("✅ 性能测试完成!")

def test_entropy_tester():
    """使用原始的EntropyTester进行测试"""
    print("\n🧪 使用原始EntropyTester进行测试")
    print("=" * 50)
    
    entropy_module = load_entropy_module()
    if not entropy_module:
        return
    
    # 创建熵源和测试器
    entropy_source = entropy_module.OptimizedEntropySource()
    tester = entropy_module.EntropyTester(entropy_source)
    
    # 启动熵源
    entropy_source.start()
    
    # 预热
    print("✓ 预热阶段 (5秒)...")
    time.sleep(5)
    
    # 运行性能测试
    print("✓ 运行性能测试...")
    results = tester.run_performance_test(total_size=5*1024*1024, chunk_size=4096)  # 5MB测试
    
    # 显示结果
    print(f"\n📊 EntropyTester测试结果:")
    perf = results['performance']
    print(f"  总字节数: {perf['total_bytes']:,} bytes ({perf['total_bytes']/1024/1024:.2f} MB)")
    print(f"  持续时间: {perf['duration']:.2f} 秒")
    print(f"  吞吐量: {perf['throughput']/1024/1024:.2f} MB/s")
    print(f"  CPU使用率: {perf['cpu_usage']:.1f}%")
    
    # NIST测试结果
    if 'sp800_90' in results:
        print(f"\n🔍 NIST测试结果:")
        summary = results['overall_summary']
        print(f"  通过测试: {summary['passed_tests']}/{summary['total_tests']}")
        print(f"  通过率: {summary['pass_rate']*100:.1f}%")
    
    entropy_source.stop()

if __name__ == "__main__":
    try:
        test_original_performance()
        test_entropy_tester()
    except KeyboardInterrupt:
        print("\n⚠️ 测试被用户中断")
    except Exception as e:
        print(f"\n❌ 测试出现异常: {e}")
        import traceback
        traceback.print_exc()
